import promptlayer
import requests

def get_api_key():
    # raise an error if the api key is not set
    if promptlayer.api_key is None:
        raise Exception(
            "Please set your PROMPTLAYER_API_KEY environment variable or set API KEY in code using 'promptlayer.api_key = <your_api_key>' "
        )
    else:
        return promptlayer.api_key

def promptlayer_api_request(function_name, provider_type, args, kwargs, tags, response, request_start_time, request_end_time, api_key):
    request_response = requests.post(
        "https://api.promptlayer.com/track-request",
        json={
            "function_name": function_name,
            "provider_type": provider_type,
            "args": args,
            "kwargs": kwargs,
            "tags": tags,
            "request_response": response,
            "request_start_time": request_start_time,
            "request_end_time": request_end_time,
            "api_key": api_key,
        },
    )
    if request_response.status_code != 200:
        raise Exception(f"Error while tracking request: {request_response.json().get('message')}")

class OpenAIGeneratorProxy:
    def __init__(self, generator, api_request_arguments):
        self.generator = generator
        self.results = []
        self.api_request_arugments = api_request_arguments
    def __iter__(self):
        return self
    
    def __next__(self):
        result = next(self.generator)
        self.results.append(result)
        if result.choices[0].finish_reason == "stop":
            promptlayer_api_request(
                self.api_request_arugments["function_name"],
                self.api_request_arugments["provider_type"],
                self.api_request_arugments["args"],
                self.api_request_arugments["kwargs"],
                self.api_request_arugments["tags"],
                self.cleaned_result(),
                self.api_request_arugments["request_start_time"],
                self.api_request_arugments["request_end_time"],
                get_api_key(),
            )
        return result

    def cleaned_result(self):
        response = ""
        for result in self.results:
            response = f'{response}{result.choices[0].text}'
        final_result = self.results[-1]
        final_result.choices[0].text = response
        return final_result



from langchain.llms import OpenAI
from typing import List, Optional
from pydantic import BaseModel
from langchain.schema import LLMResult
import datetime

class OpenAI(OpenAI, BaseModel):
    pl_tags: Optional[List[str]]

    def _generate(self, prompts: List[str], stop: Optional[List[str]] = None
    ) -> LLMResult: 
        from promptlayer.utils import get_api_key, promptlayer_api_request
        request_start_time = datetime.datetime.now().timestamp()
        generated_responses = super()._generate(prompts, stop)
        request_end_time = datetime.datetime.now().timestamp()
        for i in range(len(prompts)):
            prompt = prompts[i]
            resp = generated_responses.generations[i]
            promptlayer_api_request(
                "langchain.PromptLayerOpenAI",
                "langchain",
                [prompt],
                self._identifying_params,
                self.pl_tags,
                resp[0].text,
                request_start_time,
                request_end_time,
                get_api_key(),
            )
        return generated_responses


import datetime
import types

class PromptLayerBase(object):
    __slots__ = ["_obj", "__weakref__", "_function_name", "_provider_type"]

    def __init__(self, obj, function_name="", provider_type="openai"):
        object.__setattr__(self, "_obj", obj)
        object.__setattr__(self, "_function_name", function_name)
        object.__setattr__(self, "_provider_type", provider_type)

    def __getattr__(self, name):
        return PromptLayerBase(
            getattr(object.__getattribute__(self, "_obj"), name),
            function_name=f'{object.__getattribute__(self, "_function_name")}.{name}',
            provider_type=object.__getattribute__(self, "_provider_type"),
        )

    def __delattr__(self, name):
        delattr(object.__getattribute__(self, "_obj"), name)

    def __setattr__(self, name, value):
        setattr(object.__getattribute__(self, "_obj"), name, value)

    def __call__(self, *args, **kwargs):
        from promptlayer.utils import get_api_key, promptlayer_api_request, OpenAIGeneratorProxy
        tags = kwargs.pop("pl_tags", None)
        request_start_time = datetime.datetime.now().timestamp()
        response = object.__getattribute__(self, "_obj")(*args, **kwargs)
        request_end_time = datetime.datetime.now().timestamp()
        if isinstance(response, types.GeneratorType):
            return OpenAIGeneratorProxy(response, {
                "function_name": object.__getattribute__(self, "_function_name"),
                "provider_type": object.__getattribute__(self, "_provider_type"),
                "args": args,
                "kwargs": kwargs,
                "tags": tags,
                "request_start_time": request_start_time,
                "request_end_time": request_end_time,
            })
        promptlayer_api_request(
            object.__getattribute__(self, "_function_name"),
            object.__getattribute__(self, "_provider_type"),
            args,
            kwargs,
            tags,
            response,
            request_start_time,
            request_end_time,
            get_api_key(),
        )
        return response



from promptlayer.promptlayer import PromptLayerBase
import promptlayer.langchain as langchain
import openai
import os

api_key = os.environ.get("PROMPTLAYER_API_KEY")
openai = PromptLayerBase(openai, function_name="openai")


__all__ = [
    "api_key",
    "openai",
]



from setuptools import find_packages, setup

try:
    # read the contents of your README file
    from pathlib import Path
    this_directory = Path(__file__).parent
    long_description = (this_directory / "README.md").read_text()
except:
    long_description = ""

setup(
    name="promptlayer",
    description="PromptLayer is a package to keep track of your GPT models training",
    long_description=long_description,
    long_description_content_type='text/markdown',
    author="Magniv",
    author_email="hello@magniv.io",
    url="https://www.promptlayer.com",
    project_urls={"Documentation": "https://magniv.notion.site/Prompt-Layer-Docs-db0e6f50cacf4564a6d09824ba17a629",},
    version="0.1.65",
    py_modules=["promptlayer"],
    packages=find_packages(),
    install_requires=["requests", "openai", "langchain"],
)
